@@ -16,7 +16,7 @@
 #ifndef QGSMAPTOOLDIGITIZEFEATURE_H
 #define QGSMAPTOOLDIGITIZEFEATURE_H
 
-#include "qgsmaptoolcapture.h"
+#include "qgsmaptoolcapturelayergeometry.h"
 #include "qgis_gui.h"
 
 class QgsFeature;
@@ -28,7 +28,7 @@ class QgsFeature;
  * A signal will then be emitted.
  * \since QGIS 3.10
  */
-class GUI_EXPORT QgsMapToolDigitizeFeature : public QgsMapToolCapture
+class GUI_EXPORT QgsMapToolDigitizeFeature : public QgsMapToolCaptureLayerGeometry
 {
     Q_OBJECT
 
@@ -43,6 +43,7 @@ class GUI_EXPORT QgsMapToolDigitizeFeature : public QgsMapToolCapture
     QgsMapToolDigitizeFeature( QgsMapCanvas *canvas, QgsAdvancedDigitizingDockWidget *cadDockWidget, CaptureMode mode = QgsMapToolCapture::CaptureNone );
 
     QgsMapToolCapture::Capabilities capabilities() const override;
+    bool supportsTechnique( Qgis::CaptureTechnique technique ) const override;
 
     void cadCanvasReleaseEvent( QgsMapMouseEvent *e ) override;
 
@@ -67,7 +68,7 @@ class GUI_EXPORT QgsMapToolDigitizeFeature : public QgsMapToolCapture
      * Emitted whenever the digitizing has been ended without digitizing
      * any feature
      */
-    void digitizingFinished( );
+    void digitizingFinished();
 
   protected:
 
@@ -82,14 +83,21 @@ class GUI_EXPORT QgsMapToolDigitizeFeature : public QgsMapToolCapture
      * \since QGIS 3.0
      */
     void setCheckGeometryType( bool checkGeometryType );
+    // TODO QGIS 4: remove if GRASS plugin is dropped
 
   private:
 
     /**
      * Called when the feature has been digitized.
-     * \param f the new created feature
+     * \param geometry the digitized geometry
      */
-    virtual void digitized( const QgsFeature &f );
+    void layerGeometryCaptured( const QgsGeometry &geometry ) FINAL;
+
+    /**
+     * Called when the feature has been digitized
+     * \since QGIS 3.26
+     */
+    virtual void featureDigitized( const QgsFeature &feature )  {Q_UNUSED( feature )} SIP_FORCE
 
     /**
      * individual layer per digitizing session
